package packageForLT06;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;

/**
 * <p>
 * The class encapsulates a view of a DotModel (where a "view" is a visual
 * representation of data). This view is based on a widget container and can be
 * instantiated and displayed.
 * </p>
 * 
 * <p>
 * The view is a single container widget that contains one widget, the
 * DotDrawingPanel. This widget provides a single bounded two-dimensional plane.
 * The dots in the data model are each drawn on the plane in their specified
 * location and with their specified size. The user space is mapped directly to
 * the device space.
 * </p>
 * 
 * <p>
 * Each dot is shown as a basic wireframe.
 * </p>
 * 
 * <p>
 * The only constructor this class has is one that requires a DotModel object.
 * It doesn't make sense to create this view without its data model.
 * </p>
 * 
 * <p>
 * This view is capable of listening to data models and will install itself on
 * the passed data model. Any time the data model changes, this view will
 * repaint itself.
 * </p>
 * 
 * @author mb
 * 
 */
public class WireFrameDotView extends JFrame implements DotModelListener, DotView {
	private static int count = 0;
	private DotModel theModel;
	private int width;
	private int height;

	private JPanel thePanel;

	private WireFrameDotView() {
	}

	/**
	 * Constructs a view as specified in the class description. The view gets
	 * installed upon the passed model.
	 * 
	 * @param model
	 */
	public WireFrameDotView(DotModel model) {
		int width = model.getMaxX();
		int height = model.getMaxY();
		this.setTitle("[" + count + "]"
				+ this.getClass().getName().replaceAll(".*\\.", ""));
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(width, height);
		this.setMinimumSize(new Dimension(width, height));
		this.setBackground(Color.WHITE);

		thePanel = new DotDrawingWidget(model);
		this.add(thePanel);

		this.pack();
		this.setVisible(true);

		theModel = model;
		// does this view need to update itself in response to any changes
		// in the model? If so, install a listener here
		model.addListener(this);
		count++;

	}

	/* (non-Javadoc)
	 * @see packageForLT06.DotView#paintComponent(java.awt.Graphics)
	 */
	@Override
	public void paintComponent(Graphics g) {
		// super.paintComponent(g); // yes, this should be the first statement,
		// but the code works as-is without this potentially confusing part

		// the paintComponent method for each of the child widgets of this
		// widget will be invoked
	}

	/* (non-Javadoc)
	 * @see packageForLT06.DotView#getPreferredSize()
	 */
	@Override
	public Dimension getPreferredSize() {
		return new Dimension(width, height);
	}

	/* (non-Javadoc)
	 * @see packageForLT06.DotView#changed()
	 */
	@Override
	public void changed() {
		// what should happen if we come to learn that the model has changed? we
		// should repaint ourself.
		this.repaint();
	}

	/* (non-Javadoc)
	 * @see packageForLT06.DotView#isInTopRegion(java.awt.Point)
	 */
	@Override
	public boolean isInTopRegion(Point point) {
		boolean isInTopRegion = point.y < theModel.getMaxY() / 2;
		return isInTopRegion;
	}

}
